# -*- coding:utf-8 -*-

import argparse

from os.path import join

from .cmc_stats import load_scores_from_file, get_cmc_curve, CMCstats
from .report import generate_cmc_report
from .plot import plot_cmc_stats

__copyright__ = 'Copyright 2017'
__author__ = u'Manuel Aguado Martínez'


def get_cmc_info():
    # Setting script arguments
    ap = argparse.ArgumentParser()
    ap.add_argument("-p", "--path", required=False, default='.',
                    help="The path to the scores files")
    ap.add_argument("-ms", "--scores_filenames", required=True,
                    help="The scores file. Multiple files must be"
                         " separated by a comma")
    ap.add_argument("-t", "--true_pairs_file_names", required=True,
                    help="Genuine pairs file. Multiple files must be"
                         " separated by a comma.")
    ap.add_argument("-e", "--experiment_names", required=True,
                    help="Experiment ID. Multiple IDS must be separated by "
                         " comma")
    ap.add_argument("-r", "--maximum_rank", required=False, default=20,
                    help="The maximum rank to calculate the penetration"
                         " coefficient. (default=20)")
    ap.add_argument("-lw", "--line_width", required=False, default=2,
                    help="Line width for plots (default=2)")
    ap.add_argument("-lf", "--legend_font", required=False, default=12,
                    help="The size of the plots legend font (default=12)")
    ap.add_argument("-np", "--no_plots", required=False, action='store_true',
                    help="Indicates whether to not plot the results")
    ap.add_argument("-sp", "--save_path", required=False, default='',
                    help="Path to save the plots (if -s was specified)"
                         " and stats report")
    ap.add_argument("-pf", "--plots_format", required=False, default='png',
                    help="Format to save plots. Valid formats are:"
                         "(png, pdf, ps, eps and svg)")
    ap.add_argument("-rf", "--report_format", required=False, default='csv',
                    help="Format to save the report. Valid formats are:"
                         " (csv, html, tex, json). Default csv.")
    ap.add_argument("-sr", "--save_dpi", required=False, default=None,
                    help="Plots resolution (dots per inch). If not given"
                         " it will default to the value savefig.dpi in the"
                         " matplotlibrc file")
    ap.add_argument("-ds", "--ds_scores", required=False, action='store_true',
                    help='Indicates whether the input scores are dissimilarity'
                         'scores')
    args = ap.parse_args()

    # Parsing script arguments
    score_filenames = args.scores_filenames.split(',')
    true_pairs_filenames = args.true_pairs_file_names.split(',')
    if len(true_pairs_filenames) == 1:
        true_pairs_filenames *= len(score_filenames)
    experiment_names = args.experiment_names.split(',')
    experiments = zip(score_filenames, true_pairs_filenames, experiment_names)
    rank = int(args.maximum_rank)
    line_width = int(args.line_width)
    lgf_size = int(args.legend_font)
    ext = '.' + args.plots_format
    dpi = None if args.save_dpi is None else int(args.save_dpi)

    # Calculating CMC values for each experiment and plotting them
    stats = []
    for i, exp in enumerate(experiments):
        s_filename = join(args.path, exp[0])
        tp_filename = join(args.path, exp[1])
        experiment_name = exp[2]

        print('%s: Loading scores file...' % experiment_name)
        scores = load_scores_from_file(s_filename, tp_filename, args.ds_scores)

        print('%s: Calculating CMC cruve...' % experiment_name)
        rank_values = get_cmc_curve(scores, rank)

        stats.append(CMCstats(exp_id=experiment_name, ranks=rank_values))

    # Generating reports
    print('Generating report...')

    filename = join(args.save_path, 'pyeer_report.' + args.report_format)
    generate_cmc_report(stats, rank, filename)

    if not args.no_plots:
        print('Plotting...')
        plot_cmc_stats(stats, rank, line_width, lgf_size, True,
                       dpi, args.save_path, ext)



# -*- coding:utf-8 -*-

import csv
import pkg_resources
import json

__copyright__ = 'Copyright 2017'
__author__ = u'Bsc. Manuel Aguado Martínez'


LATEX_TABLE_CMC_MAX_RANK = 4


def generate_json_eer_report(stats, ids, save_file):
    """ Generate a JSON file with the given statistics

    @param stats: An iterable with instances of the named tuple Stats
    @type stats: iterable
    @param ids: An iterable with an ID (str) for each stat
    @type ids: iterable
    @param save_file: The filename used to save the report
    @type save_file: str
    """
    with open(save_file, 'w') as sf:
        pkg_version = pkg_resources.require('pyeer')[0].version

        jdict = {
            'Information': 'Generated using PyEER ' + pkg_version,
            'Legend:': {
                'GMean': 'Genuine scores distribution mean',
                'GSTD': 'Genuine scores distribution '
                        'standard deviation',
                'IMean': 'Impostor scores distribution mean',
                'IVariance': 'Impostor scores distribution '
                             'standard deviation',
                "Sensitivity index (d')": "See NICE:II protocol"
                                          " evaluation",
                'AUC': 'Area under the ROC curve',
                "J-Index": "Youden's J statistic (Youden's Index)",
                "MCC": "Matthews Correlation Coefficient",
                'EER': 'Equal Error Rate',
                'EERlow, EERhigh': 'See FVC2000 protocol evaluation',
                'FMR': 'False Match Rate',
                'FNMR': 'False Non-Match Rate',
                '_TH': 'Threshold',
                'EER_TH': 'Threshold for which EERlow and EERHigh were'
                          ' calculated'
            }
        }

        for i, st in enumerate(stats):
            st_dict = {
                'GMean': st.gmean,
                'GSTD': st.gstd,
                'IMean': st.imean,
                'ISTD': st.istd,
                "Sensitivity index (d')": st.decidability,
                'AUC': st.auc,
                'J-Index': st.j_index,
                'J-Index Threshold': st.j_index_th,
                'MCC': st.mccoef,
                'MCC Threshold': st.mccoef_th,
                'EERlow': st.eer_low,
                'EERhigh': st.eer_high,
                'EER': st.eer,
                'ZeroFMR': st.fmr0,
                'FMR1000': st.fmr1000,
                'FMR100': st.fmr100,
                'FMR20': st.fmr20,
                'FMR10': st.fmr10,
                'ZeroFNMR': st.fnmr0,
                'EER Threshold': st.eer_th,
                'ZeroFMR Threshold': st.fmr0_th,
                'FMR1000 Threshold': st.fmr1000_th,
                'FMR100 Threshold': st.fmr100_th,
                'FMR20 Threshold': st.fmr20_th,
                'FMR10 Threshold': st.fmr10_th,
                'ZeroFNMR Threshold': st.fnmr0_th,
            }
            jdict['Stats for %s' % ids[i]] = st_dict

        json.dump(jdict, sf, ensure_ascii=False, indent=4)


def generate_html_eer_report(stats, ids, save_file):
    """ Generate an HTML file with the given statistics

    @param stats: An iterable with instances of the named tuple Stats
    @type stats: iterable
    @param ids: An iterable with an ID (str) for each stat
    @type ids: iterable
    @param save_file: The filename used to save the report
    @type save_file: str
    """
    with open(save_file, 'w') as sf:

        # Write html tag
        sf.write('<html>\n')

        # Beginning head section
        sf.write('<head>')

        # Writing encoding type
        sf.write('<meta charset="UTF-8">\n')

        # Writing styles
        sf.write('<style>')
        sf.write('td{ padding-right: 15px; padding-left:15px;}')
        sf.write('</style>')

        # Ending head section
        sf.write('</head>')

        # Beginning body section
        sf.write('<body>')

        # Writing html table tag
        sf.write('<table>\n')

        # Writing table caption
        pkg_version = pkg_resources.require('pyeer')[0].version
        caption = 'Generated using PyEER ' + pkg_version
        sf.write('<caption><h3>%s</h3></caption>\n' % caption)

        # Writing table headers
        sf.write('<thead>\n')
        sf.write('<tr>\n')
        sf.write('<th>%s</th>\n' % 'Experiment ID')
        sf.write('<th>%s</th>\n' % 'GMean')
        sf.write('<th>%s</th>\n' % 'GSTD')
        sf.write('<th>%s</th>\n' % 'IMean')
        sf.write('<th>%s</th>\n' % 'ISTD')
        sf.write('<th>%s</th>\n' % "Sensitivity index (d')")
        sf.write('<th>%s</th>\n' % 'AUC')
        sf.write('<th>%s</th>\n' % 'J-Index')
        sf.write('<th>%s</th>\n' % 'J-Index Threshold')
        sf.write('<th>%s</th>\n' % 'MCC')
        sf.write('<th>%s</th>\n' % 'MCC Threshold')
        sf.write('<th>%s</th>\n' % 'EERlow')
        sf.write('<th>%s</th>\n' % 'EERhigh')
        sf.write('<th>%s</th>\n' % 'EER')
        sf.write('<th>%s</th>\n' % 'ZeroFMR')
        sf.write('<th>%s</th>\n' % 'FMR1000')
        sf.write('<th>%s</th>\n' % 'FMR100')
        sf.write('<th>%s</th>\n' % 'FMR20')
        sf.write('<th>%s</th>\n' % 'FMR10')
        sf.write('<th>%s</th>\n' % 'ZeroFNMR')
        sf.write('<th>%s</th>\n' % 'EER Threshold')
        sf.write('<th>%s</th>\n' % 'ZeroFMR Threshold')
        sf.write('<th>%s</th>\n' % 'FMR1000 Threshold')
        sf.write('<th>%s</th>\n' % 'FMR100 Threshold')
        sf.write('<th>%s</th>\n' % 'FMR20 Threshold')
        sf.write('<th>%s</th>\n' % 'FMR10 Threshold')
        sf.write('<th>%s</th>\n' % 'ZeroFNMR Threshold')
        sf.write('</tr>\n')
        sf.write('</thead>\n')

        # Writing table body
        sf.write('<tbody>\n')

        for i, st in enumerate(stats):
            # Writing stats
            sf.write('<tr>\n')
            sf.write('<td>%s</td>\n' % ids[i])
            sf.write('<td>%f</td>\n' % st.gmean)
            sf.write('<td>%f</td>\n' % st.gstd)
            sf.write('<td>%f</td>\n' % st.imean)
            sf.write('<td>%f</td>\n' % st.istd)
            sf.write('<td>%f</td>\n' % st.decidability)
            sf.write('<td>%f</td>\n' % st.auc)
            sf.write('<td>%f</td>\n' % st.j_index)
            sf.write('<td>%f</td>\n' % st.j_index_th)
            sf.write('<td>%f</td>\n' % st.mccoef)
            sf.write('<td>%f</td>\n' % st.mccoef_th)
            sf.write('<td>%f</td>\n' % st.eer_low)
            sf.write('<td>%f</td>\n' % st.eer_high)
            sf.write('<td>%f</td>\n' % st.eer)
            sf.write('<td>%f</td>\n' % st.fmr0)
            sf.write('<td>%f</td>\n' % st.fmr1000)
            sf.write('<td>%f</td>\n' % st.fmr100)
            sf.write('<td>%f</td>\n' % st.fmr20)
            sf.write('<td>%f</td>\n' % st.fmr10)
            sf.write('<td>%f</td>\n' % st.fnmr0)
            sf.write('<td>%f</td>\n' % st.eer_th)
            sf.write('<td>%f</td>\n' % st.fmr0_th)
            sf.write('<td>%f</td>\n' % st.fmr1000_th)
            sf.write('<td>%f</td>\n' % st.fmr100_th)
            sf.write('<td>%f</td>\n' % st.fmr20_th)
            sf.write('<td>%f</td>\n' % st.fmr10_th)
            sf.write('<td>%f</td>\n' % st.fnmr0_th)
            sf.write('<tr>\n')

        # Closing table body
        sf.write('</tbody>\n')

        # Writing table footer
        sf.write('<tfoot>\n')

        sf.write('<tr><td colspan="27"><strong>GMean:</strong> Genuine scores'
                 ' distribution  mean</td><tr>\n')
        sf.write('<tr><td colspan="27"><strong>GSTD:</strong> Genuine scores'
                 '  distribution standard deviation</td><tr>\n')
        sf.write('<tr><td colspan="27"><strong>IMean:</strong> Impostor scores'
                 ' distribution  mean</td></tr>\n')
        sf.write('<tr><td colspan="27"><strong>IVariance:</strong> Impostor'
                 ' scores distribution standard deviation</td></tr>\n')
        sf.write('<tr><td colspan="27"><strong>Sensitivity index' "(d')" ':'
                 '</strong> NICE:II protocol evaluation </td></tr>\n')
        sf.write('<tr><td colspan="27"><strong>AUC:</strong> Area under the'
                 ' ROC curve </td></tr>\n')
        sf.write('<tr><td colspan="27"><strong>J-Index:</strong> ' "Youden's J"
                 " statistic (Youden's Index) </td></tr>\n")
        sf.write('<tr><td colspan="27"><strong>MCC:</strong> Matthews'
                 ' Correlation Coefficient </td></tr>\n')
        sf.write('<tr><td colspan="27"><strong>EER:</strong> Equal Error Rate'
                 '</td></tr>\n')
        sf.write('<tr><td colspan="27"><strong>EERlow, EERhigh:</strong> See'
                 ' FVC2000 protocol evaluation </td></tr>\n')
        sf.write('<tr><td colspan="27"><strong>FMR:</strong> False Match Rate'
                 '</td></tr>\n')
        sf.write('<tr><td colspan="27"><strong>FNMR:</strong> False Non-Match'
                 ' Rate</td></tr>\n')
        sf.write('<tr><td colspan="27"><strong>EER Threshold:</strong> '
                 ' Threshold for which EERlow and EERHigh were calculated'
                 '</td></tr>\n')

        # Closing table footer
        sf.write('<tfoot>\n')

        # Closing html table tag
        sf.write('</table>\n')

        # Closing body tag
        sf.write('</body>')

        # Closing html tag
        sf.write('</html>\n')


def generate_tex_eer_report(stats, ids, save_file):
    """ Generate an TEX file with the given statistics

    @param stats: An iterable with instances of the named tuple Stats
    @type stats: iterable
    @param ids: An iterable with an ID (str) for each stat
    @type ids: iterable
    @param save_file: The filename used to save the report
    @type save_file: str
    """
    with open(save_file, 'w') as sf:

        # document type
        sf.write('\documentclass[10pt]{article}\n')

        # Writing document title
        sf.write('\\title{EER report}\n')

        # Writing author
        pkg_version = pkg_resources.require('pyeer')[0].version
        caption = 'Generated using PyEER ' + pkg_version
        sf.write('\\author{%s}\n' % caption)

        # Writing document begin
        sf.write('\\begin{document}\n')

        # Making title
        sf.write('\maketitle\n')

        # Table legend
        sf.write('\\begin{itemize}\n')
        sf.write('\item \\textbf{GMean:} Genuine scores distribution  mean\n')
        sf.write('\item \\textbf{GSTD:} Genuine scores distribution standard '
                 'deviation\n')
        sf.write('\item \\textbf{IMean:} Impostor scores distribution  mean\n')
        sf.write('\item \\textbf{IVariance:} Impostor scores distribution'
                 ' standard deviation')
        sf.write("\item \\textbf{Sensitivity index (d')}:NICE:II protocol "
                 "evaluation\n")
        sf.write('\item \\textbf{AUC:} Area under the ROC curve\n')
        sf.write("\item \\textbf{J-Index:} Youden's J statistic (Youden's "
                 "Index)\n")
        sf.write('\item \\textbf{MCC:} Matthews Correlation Coefficient\n')
        sf.write('\item \\textbf{EER:} Equal Error Rate\n')
        sf.write('\item \\textbf{EERlow,} EERhigh: See FVC2000 protocol '
                 'evaluation\n')
        sf.write('\item \\textbf{FMR:} False Match Rate\n')
        sf.write('\item \\textbf{FNMR:} False Non-Match Rate\n')
        sf.write('\item \\textbf{EER TH:} Threshold for which EERlow'
                 ' and EERHigh were calculated\n')
        sf.write('\item TH: Threshold\n')
        sf.write('\end{itemize}\n')

        # Beginning table
        sf.write('\\begin{table}\n')

        # Centering
        sf.write('\centering\n')

        # Writing table caption
        sf.write('\caption{%s.}\label{eer_table1}\n' % caption)

        # Beginning tabular block
        sf.write('\\begin{tabular}{%s}\n' % ('l' * 6))

        # Inserting line
        sf.write('\hline\n')

        # Inserting table headers
        sf.write('\\textbf{%s} ' % 'Experiment ID')
        sf.write('& \\textbf{%s} ' % 'GMean')
        sf.write('& \\textbf{%s} ' % 'GSTD')
        sf.write('& \\textbf{%s} ' % 'IMean')
        sf.write('& \\textbf{%s} ' % 'ISTD')
        sf.write('& \\textbf{%s} ' % "d'")
        sf.write('\\\\\n')

        for i, st in enumerate(stats):
            # Writing rank values
            sf.write('\hline\n')
            sf.write('%s' % ids[i])
            sf.write(' & %f' % st.gmean)
            sf.write(' & %f' % st.gstd)
            sf.write(' & %f' % st.imean)
            sf.write(' & %f' % st.istd)
            sf.write(' & %f' % st.decidability)
            sf.write('\\\\\n')

        # Ending tabular block
        sf.write('\end{tabular}\n')

        # Ending table
        sf.write('\end{table}\n')

        # Beginning table
        sf.write('\\begin{table}\n')

        # Centering
        sf.write('\centering\n')

        # Writing table caption
        sf.write('\caption{%s.}\label{eer_table2}\n' % caption)

        # Beginning tabular block
        sf.write('\\begin{tabular}{%s}\n' % ('l' * 5))

        # Inserting line
        sf.write('\hline\n')

        # Inserting table headers
        sf.write('\\textbf{%s} ' % 'Experiment ID')
        sf.write('& \\textbf{%s} ' % 'J-Index')
        sf.write('& \\textbf{%s} ' % 'J-Index TH')
        sf.write('& \\textbf{%s} ' % 'MCC')
        sf.write('& \\textbf{%s} ' % 'MCC TH')
        sf.write('\\\\\n')

        for i, st in enumerate(stats):
            # Writing rank values
            sf.write('\hline\n')
            sf.write('%s' % ids[i])
            sf.write(' & %f' % st.j_index)
            sf.write(' & %f' % st.j_index_th)
            sf.write(' & %f' % st.mccoef)
            sf.write(' & %f' % st.mccoef_th)
            sf.write('\\\\\n')

        # Ending tabular block
        sf.write('\end{tabular}\n')

        # Ending table
        sf.write('\end{table}\n')

        # Beginning table
        sf.write('\\begin{table}\n')

        # Centering
        sf.write('\centering\n')

        # Writing table caption
        sf.write('\caption{%s.}\label{eer_table3}\n' % caption)

        # Beginning tabular block
        sf.write('\\begin{tabular}{%s}\n' % ('l' * 5))

        # Inserting line
        sf.write('\hline\n')

        # Inserting table headers
        sf.write('\\textbf{%s} ' % 'Experiment ID')
        sf.write('& \\textbf{%s} ' % 'AUC')
        sf.write('& \\textbf{%s} ' % 'EERlow')
        sf.write('& \\textbf{%s} ' % 'EERhigh')
        sf.write('& \\textbf{%s} ' % 'EER')
        sf.write('\\\\\n')

        for i, st in enumerate(stats):
            # Writing rank values
            sf.write('\hline\n')
            sf.write('%s' % ids[i])
            sf.write(' & %f' % st.auc)
            sf.write(' & %f' % st.eer_low)
            sf.write(' & %f' % st.eer_high)
            sf.write(' & %f' % st.eer)
            sf.write('\\\\\n')

        # Ending tabular block
        sf.write('\end{tabular}\n')

        # Ending table
        sf.write('\end{table}\n')

        # Beginning table
        sf.write('\\begin{table}\n')

        # Centering
        sf.write('\centering\n')

        # Writing table caption
        sf.write('\caption{%s.}\label{eer_table4}\n' % caption)

        # Beginning tabular block
        sf.write('\\begin{tabular}{%s}\n' % ('l' * 5))

        # Inserting line
        sf.write('\hline\n')

        # Inserting table headers
        sf.write('\\textbf{%s} ' % 'Experiment ID')
        sf.write('& \\textbf{%s} ' % 'ZeroFMR')
        sf.write('& \\textbf{%s} ' % 'FMR1000')
        sf.write('& \\textbf{%s} ' % 'FMR100')
        sf.write('& \\textbf{%s} ' % 'FMR20')
        sf.write('\\\\\n')

        for i, st in enumerate(stats):
            # Writing rank values
            sf.write('\hline\n')
            sf.write('%s' % ids[i])
            sf.write(' & %f' % st.fmr0)
            sf.write(' & %f' % st.fmr1000)
            sf.write(' & %f' % st.fmr100)
            sf.write(' & %f' % st.fmr20)
            sf.write('\\\\\n')

        # Ending tabular block
        sf.write('\end{tabular}\n')

        # Ending table
        sf.write('\end{table}\n')

        # Beginning table
        sf.write('\\begin{table}\n')

        # Centering
        sf.write('\centering\n')

        # Writing table caption
        sf.write('\caption{%s.}\label{eer_table5}\n' % caption)

        # Beginning tabular block
        sf.write('\\begin{tabular}{%s}\n' % ('l' * 4))

        # Inserting line
        sf.write('\hline\n')

        # Inserting table headers
        sf.write('\\textbf{%s} ' % 'Experiment ID')
        sf.write('& \\textbf{%s} ' % 'FMR10')
        sf.write('& \\textbf{%s} ' % 'ZeroFNMR')
        sf.write('& \\textbf{%s} ' % 'EER TH')
        sf.write('\\\\\n')

        for i, st in enumerate(stats):
            # Writing rank values
            sf.write('\hline\n')
            sf.write('%s' % ids[i])
            sf.write(' & %f' % st.fmr10)
            sf.write(' & %f' % st.fnmr0)
            sf.write(' & %f' % st.eer_th)
            sf.write('\\\\\n')

        # Ending tabular block
        sf.write('\end{tabular}\n')

        # Ending table
        sf.write('\end{table}\n')

        # Beginning table
        sf.write('\\begin{table}\n')

        # Centering
        sf.write('\centering\n')

        # Writing table caption
        sf.write('\caption{%s.}\label{eer_table6}\n' % caption)

        # Beginning tabular block
        sf.write('\\begin{tabular}{%s}\n' % ('l' * 4))

        # Inserting line
        sf.write('\hline\n')

        # Inserting table headers
        sf.write('\\textbf{%s} ' % 'Experiment ID')
        sf.write('& \\textbf{%s} ' % 'ZeroFMR TH')
        sf.write('& \\textbf{%s} ' % 'FMR1000 TH')
        sf.write('& \\textbf{%s} ' % 'FMR100 TH')
        sf.write('\\\\\n')

        for i, st in enumerate(stats):
            # Writing rank values
            sf.write('\hline\n')
            sf.write('%s' % ids[i])
            sf.write(' & %f' % st.fmr0_th)
            sf.write(' & %f' % st.fmr1000_th)
            sf.write(' & %f' % st.fmr100_th)
            sf.write('\\\\\n')

        # Ending tabular block
        sf.write('\end{tabular}\n')

        # Ending table
        sf.write('\end{table}\n')

        # Beginning table
        sf.write('\\begin{table}\n')

        # Centering
        sf.write('\centering\n')

        # Writing table caption
        sf.write('\caption{%s.}\label{eer_table7}\n' % caption)

        # Beginning tabular block
        sf.write('\\begin{tabular}{%s}\n' % ('l' * 4))

        # Inserting line
        sf.write('\hline\n')

        # Inserting table headers
        sf.write('\\textbf{%s} ' % 'Experiment ID')
        sf.write('& \\textbf{%s} ' % 'FMR20 TH')
        sf.write('& \\textbf{%s} ' % 'FMR10 TH')
        sf.write('& \\textbf{%s} ' % 'ZeroFNMR TH')
        sf.write('\\\\\n')

        for i, st in enumerate(stats):
            # Writing rank values
            sf.write('\hline\n')
            sf.write('%s' % ids[i])
            sf.write(' & %f' % st.fmr20_th)
            sf.write(' & %f' % st.fmr10_th)
            sf.write(' & %f' % st.fnmr0_th)
            sf.write('\\\\\n')

        # Ending tabular block
        sf.write('\end{tabular}\n')

        # Ending table
        sf.write('\end{table}\n')

        # Writing document end
        sf.write('\end{document}\n')


def generate_csv_eer_report(stats, ids, save_file):
    """ Generate a CSV file with the given statistics

    @param stats: An iterable with instances of the named tuple Stats
    @type stats: iterable
    @param ids: An iterable with an ID (str) for each stat
    @type ids: iterable
    @param save_file: The filename used to save the report
    @type save_file: str
    """
    with open(save_file, 'w') as sf:

        # Writing headers
        writer = csv.writer(sf)

        # Writing package version
        pkg_version = pkg_resources.require('pyeer')[0].version
        writer.writerow(['Generated using PyEER ' + pkg_version])

        row = ['Experiment ID', 'GMean', 'GSTD', 'IMean',
               'ISTD', "Sensitivity index (d')", 'AUC', 'J-Index',
               'J-Index_TH', 'MCC', 'MCC_TH', 'EERlow',
               'EERhigh', 'EER', 'ZeroFMR', 'FMR1000', 'FMR100',
               'FMR20', 'FMR10', 'ZeroFNMR', 'EER_TH', 'ZeroFMR_TH',
               'FMR1000_TH', 'FMR100_TH', 'FMR20_TH', 'FMR10_TH',
               'ZeroFNMR_TH']
        writer.writerow(row)

        for i, st in enumerate(stats):
            # Writing stats
            row = [ids[i], st.gmean, st.gstd, st.imean, st.istd,
                   st.decidability, st.auc, st.j_index, st.j_index_th,
                   st.mccoef, st.mccoef_th, st.eer_low, st.eer_high,
                   st.eer, st.fmr0, st.fmr1000, st.fmr100, st.fmr20,
                   st.fmr10, st.fnmr0, st.eer_th, st.fmr0_th, st.fmr1000_th,
                   st.fmr100_th, st.fmr20_th, st.fmr10_th, st.fnmr0_th]
            writer.writerow(row)

        # Writing legend
        writer.writerow([])
        writer.writerow(['Legend:'])
        writer.writerow(['GMean: Genuine scores distribution mean'])
        writer.writerow(['GSTD: Genuine scores distribution '
                         'standard deviation'])
        writer.writerow(['IMean: Impostor scores distribution mean'])
        writer.writerow(['IVariance: Impostor scores distribution '
                         'standard deviation'])
        writer.writerow(["Sensitivity index (d'): See NICE:II protocol"
                         " evaluation"])
        writer.writerow(['AUC: Area under the ROC curve'])
        writer.writerow(["J-Index: Youden's J statistic (Youden's Index)"])
        writer.writerow(["MCC: Matthews Correlation Coefficient"])
        writer.writerow(['EER: Equal Error Rate'])
        writer.writerow(['EERlow, EERhigh: See FVC2000 protocol evaluation'])
        writer.writerow(['FMR: False Match Rate'])
        writer.writerow(['FNMR: False Non-Match Rate'])
        writer.writerow(['_TH: Threshold'])
        writer.writerow(['EER_TH: Threshold for which EERlow and EERHigh were'
                         ' calculated'])


def export_error_rates(fmr, fnmr, filename):
    """Exports the given error rates to a CSV file

    @param fmr: False Match Rates
    @type fmr: iterable
    @param fnmr: False Non-Match Rates
    @type fnmr: iterable
    @param filename: The output filename
    @type filename: str
    """
    with open(filename, 'w') as sf:
        # Creating CSV writer
        writer = csv.writer(sf)

        # Writing rates header
        writer.writerow(['FMR', 'FNMR'])

        # Writing rates
        for i in range(len(fmr)):
            writer.writerow([fmr[i], fnmr[i]])


def generate_eer_report(stats, ids, save_file):
    """Writes EER statistics to a file

    @param stats: An iterable with instances of the named tuple Stats
    @type stats: iterable
    @param ids: An iterable with an ID (str) for each stat
    @type ids: iterable
    @param save_file: The filename used to save the report.
        The file format will depend on the filename extension.
    @type save_file: str
    """

    # Getting file extension
    ext = save_file.split('.')[-1]

    if ext.lower() == 'csv':
        generate_csv_eer_report(stats, ids, save_file)
    elif ext.lower() == 'html':
        generate_html_eer_report(stats, ids, save_file)
    elif ext.lower() == 'tex':
        generate_tex_eer_report(stats, ids, save_file)
    elif ext.lower() == 'json':
        generate_json_eer_report(stats, ids, save_file)
    else:
        raise ValueError('Unsupported file format')


def generate_csv_cmc_report(stats, max_rank, save_file):
    """ Generates a CSV file with the given CMC rank values

    @param exps_cmc: A list of CMCstats instances
    @type exps_cmc: list
    @param max_rank: The maximum rank of the CMC curves
    @type max_rank: int
    @param save_file: The filename used to save the report
    @type save_file: str
    """
    with open(save_file, 'w') as sf:

        # Writing headers
        writer = csv.writer(sf)
        row = ['Rank-' + str(i) for i in range(1, max_rank + 1)]
        writer.writerow(['Experiment ID'] + row)

        for st in stats:
            # Writing rank values
            writer.writerow([st.exp_id] + st.ranks)


def generate_html_cmc_report(stats, max_rank, save_file):
    """ Generates an HTML file with the given CMC rank values

    @param exps_cmc: A list of CMCstats instances
    @type exps_cmc: list
    @param max_rank: The maximum rank of the CMC curves
    @type max_rank: int
    @param save_file: The filename used to save the report
    @type save_file: str
    """
    with open(save_file, 'w') as sf:

        # Write html tag
        sf.write('<html>\n')

        # Writing encoding type
        sf.write('<meta charset="UTF-8">\n')

        # Writing styles
        sf.write('<style>')
        sf.write('td{ padding-right: 15px; padding-left:15px;}')
        sf.write('</style>')

        # Writing html table tag
        sf.write('<table>\n')

        # Writing table caption
        pkg_version = pkg_resources.require('pyeer')[0].version
        caption = 'Generated using PyEER ' + pkg_version
        sf.write('<caption><h3>%s</h3></caption>\n' % caption)

        # Writing table headers
        sf.write('<thead>\n')
        sf.write('<tr>\n')
        sf.write('<th>%s</th>\n' % 'Experiment ID')
        for i in range(1, max_rank + 1):
            sf.write('<th>Rank-%d</th>\n' % i)
        sf.write('</tr>\n')
        sf.write('</thead>\n')

        # Writing table body
        sf.write('<tbody>\n')

        for st in stats:
            # Writing rank values
            sf.write('<tr>\n')
            sf.write('<td>%s</td>\n' % st.exp_id)
            for r in st.ranks:
                sf.write('<td>%f</td>\n' % r)
            sf.write('</tr>\n')

        # Closing table body
        sf.write('</tbody>\n')

        # Closing html table tag
        sf.write('</table>\n')

        # Closing html tag
        sf.write('</html>\n')


def __write_cmc_tex_table(sf, table_number, stats, max_rank):
    # Beginning table
    sf.write('\\begin{table}[h]\n')

    # Centering
    sf.write('\centering')

    # Writing table caption
    label = 'cmc_table' + str(table_number + 1)
    sf.write('\caption{CMC Ranks.}\label{%s}\n' % label)

    # Beginning tabular block
    sf.write('\\begin{tabular}{%s}\n' % ('l' * (LATEX_TABLE_CMC_MAX_RANK + 1)))

    # Inserting line
    sf.write('\hline\n')

    rstart = table_number * LATEX_TABLE_CMC_MAX_RANK
    rend = rstart + LATEX_TABLE_CMC_MAX_RANK

    # Inserting table headers
    sf.write('\\textbf{%s} ' % 'Experiment ID')
    for i in range(rstart + 1, min(max_rank, rend) + 1):
        sf.write(' & \\textbf{Rank-%d}' % i)
    sf.write('\\\\\n')

    for st in stats:
        # Writing rank values
        sf.write('\hline\n')
        sf.write('%s' % st.exp_id)
        for i in range(rstart, min(max_rank, rend)):
            sf.write(' & %f' % st.ranks[i])
        sf.write('\\\\\n')

    # Ending tabular block
    sf.write('\end{tabular}\n')

    # Ending table
    sf.write('\end{table}\n')


def generate_tex_cmc_report(stats, max_rank, save_file):
    """ Generates a TEX file with the given CMC rank values

    @param exps_cmc: A list of CMCstats instances
    @type exps_cmc: list
    @param max_rank: The maximum rank of the CMC curves
    @type max_rank: int
    @param save_file: The filename used to save the report
    @type save_file: str
    """
    with open(save_file, 'w') as sf:

        # document type
        sf.write('\documentclass[10pt]{article}\n')

        # Writing document title
        sf.write('\\title{CMC report}\n')

        # Writing author
        pkg_version = pkg_resources.require('pyeer')[0].version
        caption = 'Generated using PyEER ' + pkg_version
        sf.write('\\author{%s}\n' % caption)

        # Writing document begin
        sf.write('\\begin{document}\n')

        # Making title
        sf.write('\maketitle\n')

        table_count = int(max_rank / LATEX_TABLE_CMC_MAX_RANK)
        if max_rank % LATEX_TABLE_CMC_MAX_RANK != 0:
            table_count += 1
        for i in range(table_count):
            __write_cmc_tex_table(sf, i, stats, max_rank)

        # Writing document end
        sf.write('\end{document}\n')


def generate_json_cmc_report(stats, max_rank, save_file):
    """ Generates a JSON file with the given CMC rank values

    @param exps_cmc: A list of CMCstats instances
    @type exps_cmc: list
    @param max_rank: The maximum rank of the CMC curves
    @type max_rank: int
    @param save_file: The filename used to save the report
    @type save_file: str
    """
    with open(save_file, 'w') as sf:
        pkg_version = pkg_resources.require('pyeer')[0].version

        jdict = {
            'Information': 'Generated using PyEER ' + pkg_version,
        }

        for st in stats:
            jdict['Rank values for %s' % st.exp_id] = st.ranks

        json.dump(jdict, sf, ensure_ascii=False, indent=4)


def generate_cmc_report(stats, max_rank, save_file):
    """ Writes CMC rank values to a file

    @param exps_cmc: A list of CMCstats instances
    @type exps_cmc: list
    @param max_rank: The maximum rank of the CMC curves
    @type max_rank: int
    @param save_file: The filename used to save the report.
        The file format will depend on the filename extension.
    @type save_file: str
    """
    # Getting file extension
    ext = save_file.split('.')[-1]

    if ext.lower() == 'csv':
        generate_csv_cmc_report(stats, max_rank, save_file)
    elif ext.lower() == 'html':
        generate_html_cmc_report(stats, max_rank, save_file)
    elif ext.lower() == 'tex':
        generate_tex_cmc_report(stats, max_rank, save_file)
    elif ext.lower() == 'json':
        generate_json_cmc_report(stats, max_rank, save_file)
    else:
        raise ValueError('Unsupported file format')



from os.path import join

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.font_manager import FontProperties


STYLES = ['s--', 'v--', 'o--', '^--', ',--', '<--', '>--', '1--', '2--',
          '3--', '4--', '.--', 'p--', '*--', 'h--', 'H--', '+--', 'x--',
          'd--', '|--', '---']


def plt_det_curve(stats, ids, line_width=3, lgf_size=15, save_plots=True,
                  dpi=None, save_path='', ext='.png', resampling=True):
    """Plot the DET curve

    @param stats: An iterable with instances of the named tuple Stats
    @type stats: iterable
    @param ids: An iterable with an ID (str) for each stat
    @type ids: iterable
    @param line_width: The width of the plotted curves (default=3)
    @type line_width: int
    @param lgf_size: The size of the legend font (default=15)
    @type lgf_size: int
    @param save_plots: Indicates whether to save the plots instead
                       of showing them
    @type save_plots: bool
    @param dpi: Plots resolution (dots per inch) used when save_plots=True.
                If not given it will default to the value of savefig.dpi
                in the matplotlibrc file
    @type dpi: int
    @param save_path: Path to save the plots (if save_plots=True)
                      and stats report
    @type save_path: str
    @param ext: Format to save the plots if save_plots=True. Valid
                formats are: (.png, .pdf, .ps, .eps and .svg)
                (default='.png')
    @type ext: str
    @param resampling: Indicates whether to resample DET curves. If
        resampled, DET curves will be interpolated and sample points
        will be x-distant.
    @type ext: bool
    """
    det_fig = plt.figure()
    det_lg_fig = plt.figure()

    det_plot = det_fig.add_subplot(111)
    det_plot.set_title('DET Curves')
    det_plot.set_ylabel('FNMR')
    det_plot.set_xlabel('FMR')
    det_plot.grid(True)

    det_lg_plot = det_lg_fig.add_subplot(111)
    det_lg_plot.set_title('DET Curves (Log scale)')
    det_lg_plot.set_ylabel('FNMR')
    det_lg_plot.set_xlabel('FMR')
    det_lg_plot.set_yscale('log')
    det_lg_plot.set_xscale('log')
    det_lg_plot.grid(True, which='both', ls='--')

    for i, st in enumerate(stats):

        if resampling:
            x = np.arange(0, 1, 0.05)
            y = np.interp(x, st.fmr[::-1], st.fnmr[::-1])
        else:
            x = st.fmr
            y = st.fnmr

        # Plotting DET Curve
        det_plot.plot(
            x, y,
            STYLES[i],
            label=ids[i],
            linewidth=line_width,
            pickradius=0.5
        )
        det_lg_plot.plot(
            x, y,
            STYLES[i],
            label=ids[i],
            linewidth=line_width,
            pickradius=0.5
        )

    # Finalizing plots
    det_plot.legend(loc='best', prop=FontProperties(size=lgf_size))
    det_lg_plot.legend(loc='best', prop=FontProperties(size=lgf_size))

    # Showing plots or saving plots
    if save_plots:
        # saving plots
        det_fig.savefig(join(save_path, 'DET' + ext), dpi=dpi)
        det_lg_fig.savefig(join(save_path, 'DET (Log)' + ext), dpi=dpi)
        plt.close('all')
    else:
        plt.show()


def plt_roc_curve(stats, ids, line_width=3, lgf_size=15, save_plots=True,
                  dpi=None, save_path='', ext='.png', resampling=True):
    """Plot the ROC curve

    @param stats: An iterable with instances of the named tuple Stats
    @type stats: iterable
    @param ids: An iterable with an ID (str) for each stat
    @type ids: iterable
    @param line_width: The width of the plotted curves (default=3)
    @type line_width: int
    @param lgf_size: The size of the legend font (default=15)
    @type lgf_size: int
    @param save_plots: Indicates whether to save the plots instead
                       of showing them
    @type save_plots: bool
    @param dpi: Plots resolution (dots per inch) used when save_plots=True.
                If not given it will default to the value of savefig.dpi
                in the matplotlibrc file
    @type dpi: int
    @param save_path: Path to save the plots (if save_plots=True)
                      and stats report
    @type save_path: str
    @param ext: Format to save the plots if save_plots=True. Valid
                formats are: (.png, .pdf, .ps, .eps and .svg)
                (default='.png')
    @type ext: str
    @param resampling: Indicates whether to resample ROC curves. If
        resampled, ROC curves will be interpolated and sample points
        will be x-distant.
    @type ext: bool
    """
    roc_fig = plt.figure()
    roc_lg_fig = plt.figure()

    roc_plot = roc_fig.add_subplot(111)
    roc_plot.set_title('ROC Curves')
    roc_plot.grid(True)
    roc_plot.set_ylabel('1 - FNMR')
    roc_plot.set_xlabel('FMR')
    roc_plot.plot([0, 1], [0, 1], 'k--', linewidth=line_width)
    roc_plot.grid(True)

    roc_lg_plot = roc_lg_fig.add_subplot(111)
    roc_lg_plot.set_title('ROC Curves (Log scale)')
    roc_lg_plot.grid(True)
    roc_lg_plot.set_ylabel('1 - FNMR')
    roc_lg_plot.set_xlabel('FMR')
    roc_lg_plot.set_xscale('log')
    roc_lg_plot.grid(True, which='minor', ls='--')

    for i, st in enumerate(stats):
        # Plotting ROC Curve
        label = ids[i] + ' AUC = %f' % st.auc

        if resampling:
            x = np.arange(0, 1.05, 0.05)
            y = np.interp(x, st.fmr[::-1], (1 - st.fnmr)[::-1])
        else:
            x = st.fmr
            y = 1 - st.fnmr

        roc_plot.plot(
            x, y, STYLES[i],
            label=label,
            linewidth=line_width
        )
        roc_lg_plot.plot(
            x, y, STYLES[i],
            label=label,
            linewidth=line_width
        )

    # Finalizing plots
    roc_plot.legend(loc='best', prop=FontProperties(size=lgf_size))
    roc_lg_plot.legend(loc='best', prop=FontProperties(size=lgf_size))

    # Showing plots or saving plots
    if save_plots:
        # saving plots
        roc_fig.savefig(join(save_path, 'ROC' + ext), dpi=dpi)
        roc_lg_fig.savefig(join(save_path, 'ROC (Log)' + ext), dpi=dpi)
        plt.close('all')
    else:
        plt.show()


def plt_distributions(stats, ids, hformat=False, bins=100,
                      lgf_size=15, save_plots=True, dpi=None,
                      save_path='', ext='.png'):
    """Plot the scores distribution of each experiment

    @param stats: An iterable with instances of the named tuple Stats
    @type stats: iterable
    @param ids: An iterable with an ID (str) for each stat
    @type ids: iterable
    @param hformat: Indicates whether the impostor scores are in
                        histogram format
    @type hormat: bool
    @param bins: The number of bins to compute scores distribution
                 It will be ignored if the hist_format=True
    @type bins: int
    @param lgf_size: The size of the legend font (default=15)
    @type lgf_size: int
    @param save_plots: Indicates whether to save the plots instead
                       of showing them
    @type save_plots: bool
    @param dpi: Plots resolution (dots per inch) used when save_plots=True.
                If not given it will default to the value of savefig.dpi
                in the matplotlibrc file
    @type dpi: int
    @param save_path: Path to save the plots (if save_plots=True)
                      and stats report
    @type save_path: str
    @param ext: Format to save the plots if save_plots=True. Valid
                formats are: (.png, .pdf, .ps, .eps and .svg)
                (default='.png')
    @type ext: str
    """
    for i, st in enumerate(stats):
        # Plotting score distributions
        title = 'Score distributions experiment: ' + ids[i]
        fig, ax1 = plt.subplots()

        ax1.grid(False)
        ax1.set_xlabel('Scores')
        ax1.set_title(title)
        ax1.set_ylabel('Frequency (Genuines)')
        # ax1.tick_params(axis='y', labelcolor=color)

        ax2 = ax1.twinx()
        ax2.set_ylabel('Frequency (Impostors)')

        if hformat:
            # Obtaining histogram
            m = max(st.gen_scores)
            x = np.arange(m)
            ghist = np.histogram(st.gen_scores, bins=np.arange(m + 1))[0]

            ax1.plot(x, ghist, color='g',
                     label='Genuine scores %d' % len(st.gen_scores))

            x = np.arange(len(st.imp_scores))
            ax2.plot(x, st.imp_scores, color='r',
                     label='Impostor scores %d' % sum(st.imp_scores))
        else:
            ax1.hist(st.gen_scores, bins=bins, color='g',
                     label='Genuine scores %d' % len(st.gen_scores))
            ax2.hist(st.imp_scores, bins=bins, alpha=0.5, color='r',
                     label='Impostor scores %d' % len(st.imp_scores))

        fig.legend(prop=FontProperties(size=lgf_size), bbox_to_anchor=(1, 1),
                   bbox_transform=ax1.transAxes)
        fig.tight_layout()

        if save_plots:
            fig_name = 'Distributions (%s)' % ids[i] + ext
            fig.savefig(join(save_path, fig_name), dpi=dpi)
            plt.close('all')
        else:
            plt.show()


def plt_error_curves(stats, ids, line_width=3, lgf_size=15, save_plots=True,
                     dpi=None, save_path='', ext='.png'):
    """Plot FMR and FNMR curves for each experiment

    @param stats: An iterable with instances of the named tuple Stats
    @type stats: iterable
    @param ids: An iterable with an ID (str) for each stat
    @type ids: iterable
    @param line_width: The width of the plotted curves (default=3)
    @type line_width: int
    @param lgf_size: The size of the legend font (default=15)
    @type lgf_size: int
    @param log_plot: Indicates whether to plot the DET curves in a
                     log-log scale
    @type log_plot: bool
    @param save_plots: Indicates whether to save the plots instead
                       of showing them
    @type save_plots: bool
    @param dpi: Plots resolution (dots per inch) used when save_plots=True.
                If not given it will default to the value of savefig.dpi
                in the matplotlibrc file
    @type dpi: int
    @param save_path: Path to save the plots (if save_plots=True)
                      and stats report
    @type save_path: str
    @param ext: Format to save the plots if save_plots=True. Valid
                formats are: (.png, .pdf, .ps, .eps and .svg)
                (default='.png')
    @type ext: str
    """
    for i, st in enumerate(stats):
        # Plotting FMR and FNMR curves
        eer_fig = plt.figure()
        eer_plot = eer_fig.add_subplot(111)
        eer_plot.grid(True)
        eer_plot.set_ylabel('Error')
        eer_plot.set_xlabel('Matching Scores')
        eer_plot.set_title('FMR and FNMR Curves')
        eer_plot.plot(st.thrs, st.fmr, linewidth=line_width,
                      label=ids[i] + ' (FMR)')
        eer_plot.plot(st.thrs, st.fnmr, linewidth=line_width,
                      label=ids[i] + ' (FNMR)')
        eer_plot.legend(loc='best', prop=FontProperties(size=lgf_size))

        if save_plots:
            fname = 'FMR and FNMR curves (%s)' % ids[i] + ext
            eer_fig.savefig(join(save_path, fname), dpi=dpi)
            plt.close('all')
        else:
            plt.show()


def plot_eer_stats(stats, ids, line_width=3, hformat=False, bins=100,
                   lgf_size=15, save_plots=True, dpi=None, save_path='',
                   ext='.png', resampling=True):
    """Plot a series of graphs from the given stats

    @param stats: An iterable with instances of the named tuple Stats
    @type stats: iterable
    @param ids: An iterable with an ID (str) for each stat
    @type ids: iterable
    @param line_width: The width of the plotted curves (default=3)
    @type line_width: int
    @param hformat: Indicates whether the impostor scores are in
                        histogram format
    @type hormat: bool
    @param bins: The number of bins to compute scores distribution
                 It will be ignored if the hist_format=True
    @type bins: int
    @param lgf_size: The size of the legend font (default=15)
    @type lgf_size: int
    @param save_plots: Indicates whether to save the plots instead
                       of showing them
    @type save_plots: bool
    @param dpi: Plots resolution (dots per inch) used when save_plots=True.
                If not given it will default to the value of savefig.dpi
                in the matplotlibrc file
    @type dpi: int
    @param save_path: Path to save the plots (if save_plots=True)
                      and stats report
    @type save_path: str
    @param ext: Format to save the plots if save_plots=True. Valid
                formats are: (.png, .pdf, .ps, .eps and .svg)
                (default='.png')
    @type ext: str
    @param resampling: Indicates whether to resample ROC and DET curves.
        If resampled, curves will be interpolated and sample points
        will be x-distant.
    @type ext: bool
    """
    # Plotting the DET curve
    plt_det_curve(stats, ids, line_width, lgf_size, save_plots,
                  dpi, save_path, ext, resampling)

    # Plotting the ROC curve
    plt_roc_curve(stats, ids, line_width, lgf_size, save_plots,
                  dpi, save_path, ext, resampling)

    # Plotting scores distribution
    plt_distributions(stats, ids, hformat, bins, lgf_size, save_plots,
                      dpi, save_path, ext)

    # Plotting error curves
    plt_error_curves(stats, ids, line_width, lgf_size, save_plots,
                     dpi, save_path, ext)


def plot_cmc_stats(stats, max_rank, line_width=3, lgf_size=15,
                   save_plots=True, dpi=None, save_path='.', ext='.png'):
    """Plot a series of graphs from the given stats

    @param stats: An iterable with instances of the named tuple CMCstats
    @type stats: iterable
    @param max_rank: The maximum rank of the CMC curves
    @type max_rank: int
    @param line_width: The width of the plotted curves (default=3)
    @type line_width: int
    @param lgf_size: The size of the legend font (default=15)
    @type lgf_size: int
    @param save_plots: Indicates whether to save the plots instead
                       of showing them
    @type save_plots: bool
    @param dpi: Plots resolution (dots per inch) used when save_plots=True.
                If not given it will default to the value of savefig.dpi
                in the matplotlibrc file
    @type dpi: int
    @param save_path: Path to save the plots (if save_plots=True)
                      and stats report
    @type save_path: str
    @param ext: Format to save the plots if save_plots=True. Valid
                formats are: (.png, .pdf, .ps, .eps and .svg)
                (default='.png')
    @type ext: str
    """
    # Preparing plots
    plt.title('CMC Curves')
    plt.ylabel('Accuracy')
    plt.xlabel('Rank')
    plt.grid(True)
    plt.axis(xmin=1, xmax=max_rank)
    plt.xticks(range(1, max_rank))

    for i, st in enumerate(stats):
        plt.plot(range(1, len(st.ranks) + 1), st.ranks, STYLES[i],
                 label=st.exp_id, linewidth=line_width)

    # Finalizing plots
    plt.legend(loc='best', prop=FontProperties(size=lgf_size))

    # Showing plots or saving plots
    if save_plots:
        # saving plots
        plt.savefig(join(save_path, 'CMC' + ext), dpi=dpi)

        # closing plots
        plt.close()
    else:
        plt.show()



# -*- coding:utf-8 -*-

from setuptools import setup, find_packages

# Get the long description from the README file
with open('README.md') as f:
    readme = f.read()

setup(
    # Package name
    name='pyeer',

    # Package version
    version='0.5.6',

    entry_points={
        'console_scripts': [
            'geteerinf = pyeer.eer_info:get_eer_info_cmd',
            'getcmcinf = pyeer.cmc_info:get_cmc_info',
        ],
    },

    include_package_data=True,

    package_data={
        'pyeer': ['example_files/hist/*.txt', 'example_files/non_hist/*.txt',
                  'example_files/cmc/*.txt'],
    },

    # Included packages
    packages=find_packages(),

    # Package author information
    author=u'Manuel Aguado Martínez',

    author_email='manuelaguadomtz@gmail.com',

    url='https://github.com/manuelaguadomtz/pyeer',

    # project_urls={
    #     'Project Information': 'https://www.researchgate.net/project/'
    #                            'Python-package-to-calculate-EER-values'
    #                             '-and-probability-curves',
    #     'Source Code': 'https://github.com/manuelaguadomtz/pyeer',
    # },

    # Package requirements
    install_requires=['numpy',
                      'matplotlib'],

    # Package description
    description='A python package for biometric and binary classification '
                'systems performance evaluation',

    long_description=readme,

    long_description_content_type='text/markdown',

    keywords=['Equal Error Rate', 'False Match Rate', 'ROC', 'DET',
              'False Non-Match Rate', 'EER', 'FMR', 'FNMR', 'ZeroFNMR',
              'ZeroFMR', 'CMC', 'Biometric Systems'],

    classifiers=[
        'Intended Audience :: Science/Research',
        'Operating System :: OS Independent',
        'Topic :: Scientific/Engineering',
        'Programming Language :: Python',
        'License :: OSI Approved :: MIT License'
    ],
)
